#!/usr/bin/env python3

"""
A program that loads a natural number and checks if its digits come in
1. a descending order (for example: 731, but not 331 or 713);
2. a non-increasing order (for example: 731 or 331, but not 713).
"""

n = int(input("n = "))
t = n  # Preserve the number for the output

# In order to compare two consecutive digits, we need to remember
# the previous one in each step.

# The last digit (the variable name is explained in the loop)
left_digit = t % 10
t //= 10

# To check if ALL items we observe satisfy a condition, we
# assume that they do and look for a counter-example.
digits_decreasing = True
digits_nonincreasing = True

while t > 0:
    right_digit = left_digit
    left_digit = t % 10
    t //= 10

    # `right_digit` used to be to the right of `left_digit` in the number.
    # So, `left_digit > right_digit` means that they are in a decreasing order.
    # Therefore, `left_digit <= right_digit` is a counter-example.
    if left_digit <= right_digit:
        digits_decreasing = False
    # Similarly, `left_digit >= right_digit` means that they are in a non-increasing order.
    # Therefore, `left_digit < right_digit` is a counter-example.
    if left_digit < right_digit:
        digits_nonincreasing = False

# After the number is done, check which conditions stayed `True` (i.e., there were
# no counter-examples).
if digits_decreasing:
    print("The digits of", n, "are decreasing.")
else:
    print("The digits of", n, "are not decreasing.")
if digits_nonincreasing:
    print("The digits of", n, "are nonincreasing.")
else:
    print("The digits of", n, "are not nonincreasing (i.e., there is an increment between at least two).")

